import React, { useCallback, useEffect, useState, useRef } from 'react';
import { useDeviceType } from '@/hooks/useDeviceType';
import { animationController } from '@/lib/animation';

// Lazy load Particles only for desktop
const Particles = React.lazy(() => 
  import('react-tsparticles').then(module => ({
    default: module.default
  }))
);

// Lazy load particles-slim
const loadParticlesSlim = () => import('tsparticles-slim').then(module => module.loadSlim);

interface AnimatedNodesProps {
  isHero?: boolean;
  priority?: boolean;
}

const AnimatedNodes: React.FC<AnimatedNodesProps> = ({ 
  isHero = false,
  priority = false 
}) => {
  const [isInteractive, setIsInteractive] = useState(false);
  const [isVisible, setIsVisible] = useState(false);
  const deviceType = useDeviceType();
  const containerRef = useRef<HTMLDivElement>(null);

  // Calculate container height based on section type
  const containerHeight = isHero ? '100dvh' : '100%';
  const minHeight = isHero ? '600px' : '400px';

  useEffect(() => {
    if (deviceType === 'mobile') return;

    const initAnimation = () => {
      setIsVisible(true);
      if (isHero || priority) {
        setIsInteractive(true);
      } else {
        const timeout = setTimeout(() => {
          setIsInteractive(true);
        }, 2000);
        return () => clearTimeout(timeout);
      }
    };

    if (isHero || priority) {
      if (document.readyState !== 'loading') {
        requestIdleCallback(() => {
          animationController.queueAnimation(initAnimation, true);
        });
      } else {
        document.addEventListener('DOMContentLoaded', () => {
          requestIdleCallback(() => {
            animationController.queueAnimation(initAnimation, true);
          });
        });
      }
    } else {
      const observer = new IntersectionObserver(
        (entries) => {
          entries.forEach((entry) => {
            if (entry.isIntersecting) {
              animationController.queueAnimation(initAnimation);
              observer.disconnect();
            }
          });
        },
        { 
          threshold: 0.1,
          rootMargin: '50px 0px'
        }
      );

      if (containerRef.current) {
        observer.observe(containerRef.current);
      }

      return () => observer.disconnect();
    }
  }, [deviceType, isHero, priority]);

  const particlesInit = useCallback(async (engine: any) => {
    if (deviceType === 'desktop') {
      const loadSlim = await loadParticlesSlim();
      await loadSlim(engine);
    }
  }, [deviceType]);

  // Don't render anything on mobile
  if (deviceType === 'mobile' || !isVisible) {
    return (
      <div 
        ref={containerRef}
        className="absolute inset-0 z-0 bg-black safari-fix"
        style={{ 
          height: containerHeight,
          minHeight: minHeight
        }}
      />
    );
  }

  const particlesOptions = {
    fpsLimit: 60,
    particles: {
      number: {
        value: isHero ? 100 : 80,
        density: {
          enable: true,
          area: 800
        }
      },
      color: {
        value: "#22d3ee"
      },
      shape: {
        type: "circle"
      },
      size: {
        value: { min: 1, max: 3 }
      },
      links: {
        enable: true,
        distance: 150,
        color: "#22d3ee",
        opacity: 0.4,
        width: 1
      },
      move: {
        enable: true,
        speed: isHero ? 1.5 : 1.2,
        direction: "none",
        random: true,
        straight: false,
        outModes: {
          default: "bounce"
        }
      },
      opacity: {
        value: 0.5
      }
    },
    interactivity: {
      detect_on: "window",
      events: {
        onHover: {
          enable: isInteractive,
          mode: "grab"
        },
        resize: true
      },
      modes: {
        grab: {
          distance: 140,
          links: {
            opacity: 0.5
          }
        }
      }
    },
    background: {
      color: "#000"
    },
    detectRetina: true,
    smooth: true,
    retina_detect: true,
    motion: {
      reduce: {
        value: true,
        factor: 4
      }
    }
  };

  return (
    <div 
      ref={containerRef}
      className="absolute inset-0 z-0 safari-fix"
      style={{ 
        height: containerHeight,
        minHeight: minHeight
      }}
    >
      <React.Suspense fallback={
        <div className="min-h-[500px] flex items-center justify-center">
          <div className="w-8 h-8 border-2 border-cyan-400 border-t-transparent rounded-full animate-spin" />
        </div>
      }>
        <Particles
          id={isHero ? "hero-particles" : `section-particles-${Math.random()}`}
          init={particlesInit}
          options={particlesOptions}
          className="absolute inset-0 z-0 safari-fix"
          style={{ 
            height: containerHeight,
            minHeight: minHeight
          }}
        />
      </React.Suspense>
    </div>
  );
};

export default React.memo(AnimatedNodes);